@@ -1,16 +1,30 @@
 from __future__ import annotations
 
+import functools
 import typing as t
 
 from sqlglot import exp
 from sqlglot._typing import E
-from sqlglot.helper import ensure_list, subclasses
+from sqlglot.helper import (
+    ensure_list,
+    is_date_unit,
+    is_iso_date,
+    is_iso_datetime,
+    seq_get,
+    subclasses,
+)
 from sqlglot.optimizer.scope import Scope, traverse_scope
 from sqlglot.schema import Schema, ensure_schema
 
 if t.TYPE_CHECKING:
     B = t.TypeVar("B", bound=exp.Binary)
 
+    BinaryCoercionFunc = t.Callable[[exp.Expression, exp.Expression], exp.DataType.Type]
+    BinaryCoercions = t.Dict[
+        t.Tuple[exp.DataType.Type, exp.DataType.Type],
+        BinaryCoercionFunc,
+    ]
+
 
 def annotate_types(
     expression: E,
@@ -48,6 +62,38 @@ def _annotate_with_type_lambda(data_type: exp.DataType.Type) -> t.Callable[[Type
     return lambda self, e: self._annotate_with_type(e, data_type)
 
 
+def _coerce_date_literal(l: exp.Expression, unit: t.Optional[exp.Expression]) -> exp.DataType.Type:
+    date_text = l.name
+    is_iso_date_ = is_iso_date(date_text)
+
+    if is_iso_date_ and is_date_unit(unit):
+        return exp.DataType.Type.DATE
+
+    # An ISO date is also an ISO datetime, but not vice versa
+    if is_iso_date_ or is_iso_datetime(date_text):
+        return exp.DataType.Type.DATETIME
+
+    return exp.DataType.Type.UNKNOWN
+
+
+def _coerce_date(l: exp.Expression, unit: t.Optional[exp.Expression]) -> exp.DataType.Type:
+    if not is_date_unit(unit):
+        return exp.DataType.Type.DATETIME
+    return l.type.this if l.type else exp.DataType.Type.UNKNOWN
+
+
+def swap_args(func: BinaryCoercionFunc) -> BinaryCoercionFunc:
+    @functools.wraps(func)
+    def _swapped(l: exp.Expression, r: exp.Expression) -> exp.DataType.Type:
+        return func(r, l)
+
+    return _swapped
+
+
+def swap_all(coercions: BinaryCoercions) -> BinaryCoercions:
+    return {**coercions, **{(b, a): swap_args(func) for (a, b), func in coercions.items()}}
+
+
 class _TypeAnnotator(type):
     def __new__(cls, clsname, bases, attrs):
         klass = super().__new__(cls, clsname, bases, attrs)
@@ -104,10 +150,8 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
         exp.DataType.Type.DATE: {
             exp.CurrentDate,
             exp.Date,
-            exp.DateAdd,
+            exp.DateFromParts,
             exp.DateStrToDate,
-            exp.DateSub,
-            exp.DateTrunc,
             exp.DiToDate,
             exp.StrToDate,
             exp.TimeStrToDate,
@@ -121,6 +165,7 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
         exp.DataType.Type.DOUBLE: {
             exp.ApproxQuantile,
             exp.Avg,
+            exp.Div,
             exp.Exp,
             exp.Ln,
             exp.Log,
@@ -139,8 +184,8 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
         },
         exp.DataType.Type.INT: {
             exp.Ceil,
-            exp.DateDiff,
             exp.DatetimeDiff,
+            exp.DateDiff,
             exp.Extract,
             exp.TimestampDiff,
             exp.TimeDiff,
@@ -157,6 +202,7 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
             exp.TimeAdd,
             exp.TimeStrToTime,
             exp.TimeSub,
+            exp.Timestamp,
             exp.TimestampAdd,
             exp.TimestampSub,
             exp.UnixToTime,
@@ -176,6 +222,7 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
             exp.Initcap,
             exp.Lower,
             exp.SafeConcat,
+            exp.SafeDPipe,
             exp.Substring,
             exp.TimeToStr,
             exp.TimeToTimeStr,
@@ -187,7 +234,7 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
         },
     }
 
-    ANNOTATORS = {
+    ANNOTATORS: t.Dict = {
         **{
             expr_type: lambda self, e: self._annotate_unary(e)
             for expr_type in subclasses(exp.__name__, (exp.Unary, exp.Alias))
@@ -201,11 +248,22 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
             for data_type, expressions in TYPE_TO_EXPRESSIONS.items()
             for expr_type in expressions
         },
+        exp.Abs: lambda self, e: self._annotate_by_args(e, "this"),
         exp.Anonymous: lambda self, e: self._annotate_with_type(e, exp.DataType.Type.UNKNOWN),
+        exp.Array: lambda self, e: self._annotate_by_args(e, "expressions", array=True),
+        exp.ArrayAgg: lambda self, e: self._annotate_by_args(e, "this", array=True),
+        exp.ArrayConcat: lambda self, e: self._annotate_by_args(e, "this", "expressions"),
+        exp.Bracket: lambda self, e: self._annotate_bracket(e),
         exp.Cast: lambda self, e: self._annotate_with_type(e, e.args["to"]),
         exp.Case: lambda self, e: self._annotate_by_args(e, "default", "ifs"),
         exp.Coalesce: lambda self, e: self._annotate_by_args(e, "this", "expressions"),
         exp.DataType: lambda self, e: self._annotate_with_type(e, e.copy()),
+        exp.DateAdd: lambda self, e: self._annotate_timeunit(e),
+        exp.DateSub: lambda self, e: self._annotate_timeunit(e),
+        exp.DateTrunc: lambda self, e: self._annotate_timeunit(e),
+        exp.Distinct: lambda self, e: self._annotate_by_args(e, "expressions"),
+        exp.Div: lambda self, e: self._annotate_div(e),
+        exp.Filter: lambda self, e: self._annotate_by_args(e, "this"),
         exp.If: lambda self, e: self._annotate_by_args(e, "true", "false"),
         exp.Interval: lambda self, e: self._annotate_with_type(e, exp.DataType.Type.INTERVAL),
         exp.Least: lambda self, e: self._annotate_by_args(e, "expressions"),
@@ -214,23 +272,70 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
         exp.Max: lambda self, e: self._annotate_by_args(e, "this", "expressions"),
         exp.Min: lambda self, e: self._annotate_by_args(e, "this", "expressions"),
         exp.Null: lambda self, e: self._annotate_with_type(e, exp.DataType.Type.NULL),
+        exp.Nullif: lambda self, e: self._annotate_by_args(e, "this", "expression"),
+        exp.Slice: lambda self, e: self._annotate_with_type(e, exp.DataType.Type.UNKNOWN),
         exp.Sum: lambda self, e: self._annotate_by_args(e, "this", "expressions", promote=True),
         exp.TryCast: lambda self, e: self._annotate_with_type(e, e.args["to"]),
         exp.VarMap: lambda self, e: self._annotate_with_type(e, exp.DataType.Type.MAP),
     }
 
+    NESTED_TYPES = {
+        exp.DataType.Type.ARRAY,
+    }
+
     # Specifies what types a given type can be coerced into (autofilled)
     COERCES_TO: t.Dict[exp.DataType.Type, t.Set[exp.DataType.Type]] = {}
 
+    # Coercion functions for binary operations.
+    # Map of type pairs to a callable that takes both sides of the binary operation and returns the resulting type.
+    BINARY_COERCIONS: BinaryCoercions = {
+        **swap_all(
+            {
+                (t, exp.DataType.Type.INTERVAL): lambda l, r: _coerce_date_literal(
+                    l, r.args.get("unit")
+                )
+                for t in exp.DataType.TEXT_TYPES
+            }
+        ),
+        **swap_all(
+            {
+                # text + numeric will yield the numeric type to match most dialects' semantics
+                (text, numeric): lambda l, r: t.cast(
+                    exp.DataType.Type, l.type if l.type in exp.DataType.NUMERIC_TYPES else r.type
+                )
+                for text in exp.DataType.TEXT_TYPES
+                for numeric in exp.DataType.NUMERIC_TYPES
+            }
+        ),
+        **swap_all(
+            {
+                (exp.DataType.Type.DATE, exp.DataType.Type.INTERVAL): lambda l, r: _coerce_date(
+                    l, r.args.get("unit")
+                ),
+            }
+        ),
+    }
+
     def __init__(
         self,
         schema: Schema,
         annotators: t.Optional[t.Dict[t.Type[E], t.Callable[[TypeAnnotator, E], E]]] = None,
         coerces_to: t.Optional[t.Dict[exp.DataType.Type, t.Set[exp.DataType.Type]]] = None,
+        binary_coercions: t.Optional[BinaryCoercions] = None,
     ) -> None:
         self.schema = schema
         self.annotators = annotators or self.ANNOTATORS
         self.coerces_to = coerces_to or self.COERCES_TO
+        self.binary_coercions = binary_coercions or self.BINARY_COERCIONS
+
+        # Caches the ids of annotated sub-Expressions, to ensure we only visit them once
+        self._visited: t.Set[int] = set()
+
+    def _set_type(
+        self, expression: exp.Expression, target_type: exp.DataType | exp.DataType.Type
+    ) -> None:
+        expression.type = target_type  # type: ignore
+        self._visited.add(id(expression))
 
     def annotate(self, expression: E) -> E:
         for scope in traverse_scope(expression):
@@ -269,9 +374,9 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
 
                 source = scope.sources.get(col.table)
                 if isinstance(source, exp.Table):
-                    col.type = self.schema.get_column_type(source, col)
+                    self._set_type(col, self.schema.get_column_type(source, col))
                 elif source and col.table in selects and col.name in selects[col.table]:
-                    col.type = selects[col.table][col.name].type
+                    self._set_type(col, selects[col.table][col.name].type)
 
             # Then (possibly) annotate the remaining expressions in the scope
             self._maybe_annotate(scope.expression)
@@ -279,7 +384,7 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
         return self._maybe_annotate(expression)  # This takes care of non-traversable expressions
 
     def _maybe_annotate(self, expression: E) -> E:
-        if expression.type:
+        if id(expression) in self._visited:
             return expression  # We've already inferred the expression's type
 
         annotator = self.annotators.get(expression.__class__)
@@ -298,19 +403,22 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
 
     def _maybe_coerce(
         self, type1: exp.DataType | exp.DataType.Type, type2: exp.DataType | exp.DataType.Type
-    ) -> exp.DataType.Type:
-        # We propagate the NULL / UNKNOWN types upwards if found
-        if isinstance(type1, exp.DataType):
-            type1 = type1.this
-        if isinstance(type2, exp.DataType):
-            type2 = type2.this
+    ) -> exp.DataType | exp.DataType.Type:
+        type1_value = type1.this if isinstance(type1, exp.DataType) else type1
+        type2_value = type2.this if isinstance(type2, exp.DataType) else type2
 
-        if exp.DataType.Type.NULL in (type1, type2):
+        # We propagate the NULL / UNKNOWN types upwards if found
+        if exp.DataType.Type.NULL in (type1_value, type2_value):
             return exp.DataType.Type.NULL
-        if exp.DataType.Type.UNKNOWN in (type1, type2):
+        if exp.DataType.Type.UNKNOWN in (type1_value, type2_value):
             return exp.DataType.Type.UNKNOWN
 
-        return type2 if type2 in self.coerces_to.get(type1, {}) else type1  # type: ignore
+        if type1_value in self.NESTED_TYPES:
+            return type1
+        if type2_value in self.NESTED_TYPES:
+            return type2
+
+        return type2_value if type2_value in self.coerces_to.get(type1_value, {}) else type1_value  # type: ignore
 
     # Note: the following "no_type_check" decorators were added because mypy was yelling due
     # to assigning Type values to expression.type (since its getter returns Optional[DataType]).
@@ -320,22 +428,25 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
     def _annotate_binary(self, expression: B) -> B:
         self._annotate_args(expression)
 
-        left_type = expression.left.type.this
-        right_type = expression.right.type.this
+        left, right = expression.left, expression.right
+        left_type, right_type = left.type.this, right.type.this
 
         if isinstance(expression, exp.Connector):
             if left_type == exp.DataType.Type.NULL and right_type == exp.DataType.Type.NULL:
-                expression.type = exp.DataType.Type.NULL
+                self._set_type(expression, exp.DataType.Type.NULL)
             elif exp.DataType.Type.NULL in (left_type, right_type):
-                expression.type = exp.DataType.build(
-                    "NULLABLE", expressions=exp.DataType.build("BOOLEAN")
+                self._set_type(
+                    expression,
+                    exp.DataType.build("NULLABLE", expressions=exp.DataType.build("BOOLEAN")),
                 )
             else:
-                expression.type = exp.DataType.Type.BOOLEAN
+                self._set_type(expression, exp.DataType.Type.BOOLEAN)
         elif isinstance(expression, exp.Predicate):
-            expression.type = exp.DataType.Type.BOOLEAN
+            self._set_type(expression, exp.DataType.Type.BOOLEAN)
+        elif (left_type, right_type) in self.binary_coercions:
+            self._set_type(expression, self.binary_coercions[(left_type, right_type)](left, right))
         else:
-            expression.type = self._maybe_coerce(left_type, right_type)
+            self._set_type(expression, self._maybe_coerce(left_type, right_type))
 
         return expression
 
@@ -344,30 +455,32 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
         self._annotate_args(expression)
 
         if isinstance(expression, exp.Condition) and not isinstance(expression, exp.Paren):
-            expression.type = exp.DataType.Type.BOOLEAN
+            self._set_type(expression, exp.DataType.Type.BOOLEAN)
         else:
-            expression.type = expression.this.type
+            self._set_type(expression, expression.this.type)
 
         return expression
 
     @t.no_type_check
     def _annotate_literal(self, expression: exp.Literal) -> exp.Literal:
         if expression.is_string:
-            expression.type = exp.DataType.Type.VARCHAR
+            self._set_type(expression, exp.DataType.Type.VARCHAR)
         elif expression.is_int:
-            expression.type = exp.DataType.Type.INT
+            self._set_type(expression, exp.DataType.Type.INT)
         else:
-            expression.type = exp.DataType.Type.DOUBLE
+            self._set_type(expression, exp.DataType.Type.DOUBLE)
 
         return expression
 
     @t.no_type_check
     def _annotate_with_type(self, expression: E, target_type: exp.DataType.Type) -> E:
-        expression.type = target_type
+        self._set_type(expression, target_type)
         return self._annotate_args(expression)
 
     @t.no_type_check
-    def _annotate_by_args(self, expression: E, *args: str, promote: bool = False) -> E:
+    def _annotate_by_args(
+        self, expression: E, *args: str, promote: bool = False, array: bool = False
+    ) -> E:
         self._annotate_args(expression)
 
         expressions: t.List[exp.Expression] = []
@@ -379,12 +492,72 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
         for expr in expressions:
             last_datatype = self._maybe_coerce(last_datatype or expr.type, expr.type)
 
-        expression.type = last_datatype or exp.DataType.Type.UNKNOWN
+        self._set_type(expression, last_datatype or exp.DataType.Type.UNKNOWN)
 
         if promote:
             if expression.type.this in exp.DataType.INTEGER_TYPES:
-                expression.type = exp.DataType.Type.BIGINT
+                self._set_type(expression, exp.DataType.Type.BIGINT)
             elif expression.type.this in exp.DataType.FLOAT_TYPES:
-                expression.type = exp.DataType.Type.DOUBLE
+                self._set_type(expression, exp.DataType.Type.DOUBLE)
+
+        if array:
+            self._set_type(
+                expression,
+                exp.DataType(
+                    this=exp.DataType.Type.ARRAY, expressions=[expression.type], nested=True
+                ),
+            )
+
+        return expression
+
+    def _annotate_timeunit(
+        self, expression: exp.TimeUnit | exp.DateTrunc
+    ) -> exp.TimeUnit | exp.DateTrunc:
+        self._annotate_args(expression)
+
+        if expression.this.type.this in exp.DataType.TEXT_TYPES:
+            datatype = _coerce_date_literal(expression.this, expression.unit)
+        elif expression.this.type.this in exp.DataType.TEMPORAL_TYPES:
+            datatype = _coerce_date(expression.this, expression.unit)
+        else:
+            datatype = exp.DataType.Type.UNKNOWN
+
+        self._set_type(expression, datatype)
+        return expression
+
+    def _annotate_bracket(self, expression: exp.Bracket) -> exp.Bracket:
+        self._annotate_args(expression)
+
+        bracket_arg = expression.expressions[0]
+        this = expression.this
+
+        if isinstance(bracket_arg, exp.Slice):
+            self._set_type(expression, this.type)
+        elif this.type.is_type(exp.DataType.Type.ARRAY):
+            contained_type = seq_get(this.type.expressions, 0) or exp.DataType.Type.UNKNOWN
+            self._set_type(expression, contained_type)
+        elif isinstance(this, (exp.Map, exp.VarMap)) and bracket_arg in this.keys:
+            index = this.keys.index(bracket_arg)
+            value = seq_get(this.values, index)
+            value_type = value.type if value else exp.DataType.Type.UNKNOWN
+            self._set_type(expression, value_type or exp.DataType.Type.UNKNOWN)
+        else:
+            self._set_type(expression, exp.DataType.Type.UNKNOWN)
+
+        return expression
+
+    def _annotate_div(self, expression: exp.Div) -> exp.Div:
+        self._annotate_args(expression)
+
+        left_type, right_type = expression.left.type.this, expression.right.type.this  # type: ignore
+
+        if (
+            expression.args.get("typed")
+            and left_type in exp.DataType.INTEGER_TYPES
+            and right_type in exp.DataType.INTEGER_TYPES
+        ):
+            self._set_type(expression, exp.DataType.Type.BIGINT)
+        else:
+            self._set_type(expression, self._maybe_coerce(left_type, right_type))
 
         return expression
