@@ -1,8 +1,11 @@
+from __future__ import annotations
+
 import itertools
 import typing as t
 
 from sqlglot import alias, exp
 from sqlglot._typing import E
+from sqlglot.dialects.dialect import DialectType
 from sqlglot.helper import csv_reader, name_sequence
 from sqlglot.optimizer.scope import Scope, traverse_scope
 from sqlglot.schema import Schema
@@ -10,12 +13,14 @@ from sqlglot.schema import Schema
 
 def qualify_tables(
     expression: E,
-    db: t.Optional[str] = None,
-    catalog: t.Optional[str] = None,
+    db: t.Optional[str | exp.Identifier] = None,
+    catalog: t.Optional[str | exp.Identifier] = None,
     schema: t.Optional[Schema] = None,
+    dialect: DialectType = None,
 ) -> E:
     """
-    Rewrite sqlglot AST to have fully qualified, unnested tables.
+    Rewrite sqlglot AST to have fully qualified tables. Join constructs such as
+    (t1 JOIN t2) AS t will be expanded into (SELECT * FROM t1 AS t1, t2 AS t2) AS t.
 
     Examples:
         >>> import sqlglot
@@ -23,32 +28,33 @@ def qualify_tables(
         >>> qualify_tables(expression, db="db").sql()
         'SELECT 1 FROM db.tbl AS tbl'
         >>>
-        >>> expression = sqlglot.parse_one("SELECT * FROM (tbl)")
+        >>> expression = sqlglot.parse_one("SELECT 1 FROM (t1 JOIN t2) AS t")
         >>> qualify_tables(expression).sql()
-        'SELECT * FROM tbl AS tbl'
-        >>>
-        >>> expression = sqlglot.parse_one("SELECT * FROM (tbl1 JOIN tbl2 ON id1 = id2)")
-        >>> qualify_tables(expression).sql()
-        'SELECT * FROM tbl1 AS tbl1 JOIN tbl2 AS tbl2 ON id1 = id2'
-
-    Note:
-        This rule effectively enforces a left-to-right join order, since all joins
-        are unnested. This means that the optimizer doesn't necessarily preserve the
-        original join order, e.g. when parentheses are used to specify it explicitly.
+        'SELECT 1 FROM (SELECT * FROM t1 AS t1, t2 AS t2) AS t'
 
     Args:
         expression: Expression to qualify
         db: Database name
         catalog: Catalog name
         schema: A schema to populate
+        dialect: The dialect to parse catalog and schema into.
 
     Returns:
         The qualified expression.
     """
     next_alias_name = name_sequence("_q_")
+    db = exp.parse_identifier(db, dialect=dialect) if db else None
+    catalog = exp.parse_identifier(catalog, dialect=dialect) if catalog else None
 
     for scope in traverse_scope(expression):
         for derived_table in itertools.chain(scope.ctes, scope.derived_tables):
+            if isinstance(derived_table, exp.Subquery):
+                unnested = derived_table.unnest()
+                if isinstance(unnested, exp.Table):
+                    joins = unnested.args.pop("joins", None)
+                    derived_table.this.replace(exp.select("*").from_(unnested.copy(), copy=False))
+                    derived_table.this.set("joins", joins)
+
             if not derived_table.args.get("alias"):
                 alias_ = next_alias_name()
                 derived_table.set("alias", exp.TableAlias(this=exp.to_identifier(alias_)))
@@ -62,23 +68,13 @@ def qualify_tables(
             if isinstance(source, exp.Table):
                 if isinstance(source.this, exp.Identifier):
                     if not source.args.get("db"):
-                        source.set("db", exp.to_identifier(db))
-                    if not source.args.get("catalog"):
-                        source.set("catalog", exp.to_identifier(catalog))
-
-                # Unnest joins attached in tables by appending them to the closest query
-                for join in source.args.get("joins") or []:
-                    scope.expression.append("joins", join)
-
-                source.set("joins", None)
-                source.set("wrapped", None)
+                        source.set("db", db)
+                    if not source.args.get("catalog") and source.args.get("db"):
+                        source.set("catalog", catalog)
 
                 if not source.alias:
-                    source = source.replace(
-                        alias(
-                            source, name or source.name or next_alias_name(), copy=True, table=True
-                        )
-                    )
+                    # Mutates the source by attaching an alias to it
+                    alias(source, name or source.name or next_alias_name(), copy=False, table=True)
 
                 pivots = source.args.get("pivots")
                 if pivots and not pivots[0].alias:
@@ -91,7 +87,9 @@ def qualify_tables(
                         header = next(reader)
                         columns = next(reader)
                         schema.add_table(
-                            source, {k: type(v).__name__ for k, v in zip(header, columns)}
+                            source,
+                            {k: type(v).__name__ for k, v in zip(header, columns)},
+                            match_depth=False,
                         )
             elif isinstance(source, Scope) and source.is_udtf:
                 udtf = source.expression
